<?php
    class Recaptchav2 extends Modules implements CaptchaProvider {

        static function __install() {
            Config::current()->set("module_recaptchav2", array("site_key" => "", "secret_key" => "", "recaptchav2_debug" => false, "recaptchav2_version" => "1.23"));
        }

        static function __uninstall() {
            Config::current()->remove("module_recaptchav2");
        }

        static function generateCaptcha() {
            $config   = Config::current();
            $settings = $config->module_recaptchav2;
            $regexp   = '/^[a-zA-Z\d-_]+$/';


            $html = '<div class="g-recaptcha" data-sitekey="'.$settings["site_key"].'"></div>'."\n";
            $html .= '<script src="https://www.google.com/recaptcha/api.js" async defer></script>'."\n";

            if (preg_match($regexp, $settings["site_key"]) && strlen($settings["site_key"]) > 10)
                if (preg_match($regexp, $settings["secret_key"]) && strlen($settings["secret_key"]) > 10)
                    return $html;

            return false;
        }

        static function checkCaptcha() {
            $config   = Config::current();
            $settings = $config->module_recaptchav2;
            $regexp   = '/^[a-zA-Z\d-_]+$/';

            $user_response = isset($_POST['g-recaptcha-response']) ? $_POST['g-recaptcha-response'] : false;

            if (!$user_response) return false;

            if (preg_match($regexp, $settings["secret_key"]) && strlen($settings["secret_key"]) > 10)
            {
                $fields_string = "";
                $fields = array('secret' => $settings["secret_key"], 'response' => $user_response);

                foreach($fields as $key => $value)
                    $fields_string .= $key .'='.$value.'&';

                $fields_string = rtrim($fields_string, '&');

                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, 'https://www.google.com/recaptcha/api/siteverify');
                curl_setopt($ch, CURLOPT_POST, count($fields));
                curl_setopt($ch, CURLOPT_POSTFIELDS, $fields_string);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);

                $result = curl_exec($ch);

                if(curl_errno($ch)){
                    error('Curl Request Error: ' . curl_error($ch));
                    if (Visitor::current()->group->can("change_settings"))
                      Flash::warning(__('Curl Request Error: ' . curl_error($ch)));
                    return false;
                }

                curl_close($ch);

                if($settings["recaptchav2_debug"]) Flash::warning(__($result));

                $response = json_decode($result, true);

                if($response["success"]) return true;
            }

            return false;
        }

        public function stylesheets($styles) {
            $styles[] = Config::current()->chyrp_url."/modules/recaptchav2/recaptchav2.css";
            return $styles;
        }

        /* ADMIN PAGES */
        public function admin_recaptchav2_settings($admin) {
            $config = Config::current();

            if (!Visitor::current()->group->can("change_settings"))
                show_403(__("Access Denied"),
                         __("You do not have sufficient privileges to change settings."));

            if (!extension_loaded('curl')) {
                Flash::warning(__("Warning: reCAPTCHA v2 requires the Curl PHP Extension."));
            }

            if (empty($_POST))
                return $admin->display("pages".DIR."recaptchav2_settings");

            if (!isset($_POST['hash']) or !authenticate($_POST['hash']))
                show_403(__("Access Denied"), __("Invalid authentication token."));

            $config->set("module_recaptchav2",
                         array(
                            "site_key"   => $_POST['site_key'],
                            "secret_key" => $_POST['secret_key'],
                            "recaptchav2_debug" => isset($_POST["recaptchav2_debug"]),
                            "recaptchav2_version" => $_POST["recaptchav2_version"]
                         ));

            fallback($_POST['site_key'], "");
            fallback($_POST['secret_key'], "");
            fallback($_POST['recaptchav2_debug'], false);
            fallback($_POST['recaptchav2_version'], 0);

            Flash::notice(__("Settings updated."), "recaptchav2_settings");
        }

        public function settings_nav($navs) {
            if (Visitor::current()->group->can("change_settings"))
                $navs["recaptchav2_settings"] = array("title" => __("reCAPTUREv2", "recaptchav2"));

            return $navs;
        }
    }
